@@ -52,9 +52,8 @@ string Complete::act( const string &str )
     for ( Actions::iterator it = actions.begin();
 	  it != actions.end();
 	  it++ ) {
-      Action *act = *it;
-      act->act_on_terminal( &terminal );
-      delete act;
+      Action &act = **it;
+      act.act_on_terminal( &terminal );
     }
     actions.clear();
   }
@@ -62,10 +61,10 @@ string Complete::act( const string &str )
   return terminal.read_octets_to_host();
 }
 
-string Complete::act( const Action *act )
+string Complete::act( const Action &act )
 {
   /* apply action to terminal */
-  act->act_on_terminal( &terminal );
+  act.act_on_terminal( &terminal );
   return terminal.read_octets_to_host();
 }
 
@@ -112,9 +111,8 @@ void Complete::apply_string( const string & diff )
       string terminal_to_host = act( input.instruction( i ).GetExtension( hostbytes ).hoststring() );
       assert( terminal_to_host.empty() ); /* server never interrogates client terminal */
     } else if ( input.instruction( i ).HasExtension( resize ) ) {
-      Resize new_size( input.instruction( i ).GetExtension( resize ).width(),
-		       input.instruction( i ).GetExtension( resize ).height() );
-      act( &new_size );
+      act( Resize( input.instruction( i ).GetExtension( resize ).width(),
+				      input.instruction( i ).GetExtension( resize ).height() ) );
     } else if ( input.instruction( i ).HasExtension( echoack ) ) {
       uint64_t inst_echo_ack_num = input.instruction( i ).GetExtension( echoack ).echo_ack_num();
       assert( inst_echo_ack_num >= echo_ack );
@@ -129,11 +127,6 @@ bool Complete::operator==( Complete const &x ) const
   return (terminal == x.terminal) && (echo_ack == x.echo_ack);
 }
 
-static bool old_ack(uint64_t newest_echo_ack, const pair<uint64_t, uint64_t> p)
-{
-  return p.first < newest_echo_ack;
-}
-
 bool Complete::set_echo_ack( uint64_t now )
 {
   bool ret = false;
@@ -147,7 +140,15 @@ bool Complete::set_echo_ack( uint64_t now )
     }
   }
 
-  input_history.remove_if( bind1st( ptr_fun( old_ack ), newest_echo_ack ) );
+  for ( input_history_type::iterator i = input_history.begin();
+	i != input_history.end(); ) {
+    input_history_type::iterator i_next = i;
+    i_next++;
+    if ( i->first < newest_echo_ack ) {
+      input_history.erase( i );
+    }
+    i = i_next;
+  }
 
   if ( echo_ack != newest_echo_ack ) {
     ret = true;
@@ -160,7 +161,7 @@ bool Complete::set_echo_ack( uint64_t now )
 
 void Complete::register_input_frame( uint64_t n, uint64_t now )
 {
-  input_history.push_back( make_pair( n, now ) );
+  input_history.push_back( std::make_pair( n, now ) );
 }
 
 int Complete::wait_time( uint64_t now ) const
@@ -175,9 +176,8 @@ int Complete::wait_time( uint64_t now ) const
   uint64_t next_echo_ack_time = it->second + ECHO_TIMEOUT;
   if ( next_echo_ack_time <= now ) {
     return 0;
-  } else {
-    return next_echo_ack_time - now;
   }
+  return next_echo_ack_time - now;
 }
 
 bool Complete::compare( const Complete &other ) const
