@@ -62,7 +62,7 @@
 
 using namespace Terminal;
 
-bool Display::ti_flag( const char *capname )
+static bool ti_flag( const char *capname )
 {
   int val = tigetflag( const_cast<char *>( capname ) );
   if ( val == -1 ) {
@@ -71,16 +71,7 @@ bool Display::ti_flag( const char *capname )
   return val;
 }
 
-int Display::ti_num( const char *capname )
-{
-  int val = tigetnum( const_cast<char *>( capname ) );
-  if ( val == -2 ) {
-    throw std::invalid_argument( std::string( "Invalid terminfo numeric capability " ) + capname );
-  }
-  return val;
-}
-
-const char *Display::ti_str( const char *capname )
+static const char *ti_str( const char *capname )
 {
   const char *val = tigetstr( const_cast<char *>( capname ) );
   if ( val == (const char *)-1 ) {
@@ -90,7 +81,7 @@ const char *Display::ti_str( const char *capname )
 }
 
 Display::Display( bool use_environment )
-  : has_ech( true ), has_bce( true ), has_title( true ), posterize_colors( false ), smcup( NULL ), rmcup( NULL )
+  : has_ech( true ), has_bce( true ), has_title( true ), smcup( NULL ), rmcup( NULL )
 {
   if ( use_environment ) {
     int errret = -2;
@@ -121,10 +112,9 @@ Display::Display( bool use_environment )
 
     /* Check if we can set the window title and icon name.  terminfo does not
        have reliable information on this, so we hardcode a whitelist of
-       terminal type prefixes.  This is the list from Debian's default
-       screenrc, plus "screen" itself (which also covers tmux). */
+       terminal type prefixes. */
     static const char * const title_term_types[] = {
-      "xterm", "rxvt", "kterm", "Eterm", "screen"
+      "xterm", "rxvt", "kterm", "Eterm", "alacritty", "screen", "tmux"
     };
 
     has_title = false;
@@ -141,12 +131,6 @@ Display::Display( bool use_environment )
       }
     }
 
-    /* posterization disabled because server now only advertises
-       xterm-256color when client has colors = 256 */
-    /*
-    posterize_colors = ti_num( "colors" ) < 256;
-    */
-
     if ( !getenv( "MOSH_NO_TERM_INIT" ) ) {
       smcup = ti_str("smcup");
       rmcup = ti_str("rmcup");
